package AlgorithmTools;

import MeasurementTools.MeasurementInfoTools.MeasurementInformation;
import MeasurementTools.MeasurementInfoTools.ListOfMeasures;
import FileManagement.FileTools.AlgorithmFileType;
import FileManagement.FileTools.AnalysisFileManager;
import FileManagement.FileTools.AnalysisFileType;
import FileManagement.FileTools.ResultFileManager;
import FileManagement.FileTools.ResultFileType;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import org.apache.commons.io.FilenameUtils;

/**<html>This  class allows the running of an algorithm that is in the repository 
 * 
 * Example:- 
 * 
 * // get an algorithm from the repository<br>
 * AlgorithmFileType algorithm = AlgorithmFileManager.getFile(numberOfAlgorithmToUse);<br>
 * <br>
 * // get a dataset from the repository<br>
 * DatasetFileManager inDataset = DatasetFileManager.getFile(numberOfFileToUse);<br>
 * <br>
 * // Create a new resultFile to place the anonymised output<br>
 * ResultFiletype outDataset = new ResultFileType(inDataset.getNumber,outDatasetName,inDataset.getDelimiter(),"Description");<br>
 * <br>
 * //create a string array of arguments you will need to know the number and type of arguments.<br>
 * // the first argument is always the inDataset location, and the second argument is the outDataset location <br>
 * String [] arguments  = {inDataset.fileLocation(), outDatsetFileLocation(), K value}<br>
 * <br>
 * RunAlgorithm runThread = new RunAlgorithm(arguments, algorithm, outDataset);<br>
 * runThread.run();<br>
 * <br>
 * the outDataset will be saved in the repository location  default C:\PPAlgTestToolData\ </html>
 * 
 */
public class RunAlgorithm implements Runnable {

    private String[] algorithmArguments;
    private AlgorithmFileType algorithm;
    private ResultFileType resultFile;

    /**
     * 
     * @param inAlgorithmArguments
     * @param inAlgorithm
     * @param inResultFile
     */
    public RunAlgorithm(String[] inAlgorithmArguments, AlgorithmFileType inAlgorithm, ResultFileType inResultFile) {
        algorithmArguments = inAlgorithmArguments;
        algorithm = inAlgorithm;
        resultFile = inResultFile;
    }

    /**
     * 
     * @return 
     */
    private String setCommandLineString() {
        StringBuilder string = new StringBuilder();
        string.append("java -jar ");
        string.append("\"");
        string.append(algorithm.getFilePath());
        string.append("\"");
        for (int i = 0; i < algorithmArguments.length; i++) {
            string.append(" ");
            string.append(algorithmArguments[i]);
        }
        return string.toString();
    }

    /**
     * 
     */
    private void runAlgorithm() {
        long runningtime;
        long finish = 0;
        long start = 0;
        try {
            String command = setCommandLineString();
            start = System.currentTimeMillis();
            Process pr = Runtime.getRuntime().exec(command);
            BufferedReader input = new BufferedReader(new InputStreamReader(pr.getInputStream()));
            String line = null;
            while ((line = input.readLine()) != null) {
                System.out.println(line);
            }
            finish = System.currentTimeMillis();
            runningtime = finish - start;
            System.out.println("Running Time:" + runningtime + "ms");
            int[] temp  = new int[1];
            temp[0] = 0;
            MeasurementInformation info = new MeasurementInformation(temp);
            info.addMeasure(ListOfMeasures.TIMETORUN, (float) runningtime);
            String filename = FilenameUtils.removeExtension(resultFile.getFileName()) + "analysis.txt";
            AnalysisFileType infoFile = new AnalysisFileType(filename, resultFile.getFileNum());
            resultFile.setAnalysisFileNumber(infoFile.getFileNum());
            try {
                info.saveFile(infoFile.getFilePath());
            } catch (IOException ex) {
            }
            AnalysisFileManager.addFileToList(infoFile);
            ResultFileManager.addFileInfoToList(resultFile);
        } catch (Exception ex) {
        } finally {
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        runAlgorithm();
    }
}
