package FileManagement.FileTools;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import org.w3c.dom.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.xml.sax.SAXException;

/**
 *
 * @author Matty
 */
public class AlgorithmFileManager {

    private final static Lock lock = new ReentrantLock();

    /**
     * 
     * @return
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException 
     */
    private static ArrayList<AlgorithmFileType> algorithmFileList() throws ParserConfigurationException, SAXException, IOException {
        do {
            if (lock.tryLock()) {
                DocumentBuilder builder;
                DocumentBuilderFactory fact;
                File file;
                Document document;
                Integer algorithmNumber = null;
                String algorithmName = null;
                String description = null;
                String inputFileType = null;
                Integer numberOfParameters = null;
                ArrayList<AlgorithmFileType> fileList = new ArrayList<>();
                file = new File(FileLocations.getAppDataFileLocation() + "AlgorithmList.xml");
                fact = DocumentBuilderFactory.newInstance();
                builder = fact.newDocumentBuilder();
                if (file.isFile()) {
                    document = builder.parse(file);
                    document.getDocumentElement().normalize();
                    NodeList algorithmListNodes = document.getElementsByTagName("Algorithm");
                    for (int i = 0; i < algorithmListNodes.getLength(); i++) {
                        Node firstAlgorithmNode = algorithmListNodes.item(i);
                        if (firstAlgorithmNode.getNodeType() == Node.ELEMENT_NODE) {
                            Element firstAlgorithmElement = (Element) algorithmListNodes.item(i);

                            NodeList algorithmIDNodeList = firstAlgorithmElement.getElementsByTagName("AlgorithmID");
                            Element algorithmIDNodeElement = (Element) algorithmIDNodeList.item(0);
                            algorithmNumber = Integer.parseInt(algorithmIDNodeElement.getAttribute("ID"));

                            NodeList algorithmNameNodeList = firstAlgorithmElement.getElementsByTagName("AlgorithmName");
                            Element algorithmNameNodeElement = (Element) algorithmNameNodeList.item(0);
                            algorithmName = algorithmNameNodeElement.getTextContent();

                            NodeList algorithmDescriptionNodeList = firstAlgorithmElement.getElementsByTagName("Description");
                            Element algorithmDescriptionNodeElement = (Element) algorithmDescriptionNodeList.item(0);
                            description = algorithmDescriptionNodeElement.getTextContent();

                            NodeList algorithmInputFileTypeNodeList = firstAlgorithmElement.getElementsByTagName("InputFileType");
                            Element algorithmInputFileTypeNodeElement = (Element) algorithmInputFileTypeNodeList.item(0);
                            inputFileType = algorithmInputFileTypeNodeElement.getTextContent();

                            NodeList algorithmNumberOfParametersNodeList = firstAlgorithmElement.getElementsByTagName("NumberOfParameters");
                            Element algorithmNumberOfParametersNodeElement = (Element) algorithmNumberOfParametersNodeList.item(0);
                            numberOfParameters = Integer.parseInt(algorithmNumberOfParametersNodeElement.getAttribute("Num"));

                            ArrayList<ArrayList<String>> parameterDescription = new ArrayList<>();
                            NodeList parameterNodeList = firstAlgorithmElement.getElementsByTagName("ParameterTypeAndDescription");
                            for (int j = 0; j < parameterNodeList.getLength(); j++) {
                                Node parameterTypeAndDescriptionNode = parameterNodeList.item(j);
                                if (parameterTypeAndDescriptionNode.getNodeType() == Node.ELEMENT_NODE) {
                                    Element parameterTypeAndDescriptionElement = (Element) parameterTypeAndDescriptionNode;
                                    String parameterDesc = parameterTypeAndDescriptionElement.getTextContent();
                                    String parameterType = parameterTypeAndDescriptionElement.getAttribute("Type");
                                    ArrayList<String> tempList = new ArrayList<>();
                                    tempList.add(parameterType);
                                    tempList.add(parameterDesc);
                                    parameterDescription.add(tempList);
                                }
                            }

                            AlgorithmFileType temp = new AlgorithmFileType(algorithmNumber, algorithmName, description, inputFileType, numberOfParameters, parameterDescription);
                            if (new File(temp.getFilePath()).isFile()) {
                                fileList.add(temp);
                            }
                        }
                    }
                }
                lock.unlock();
                return fileList;
            }
        } while (true);
    }

    /**
     * 
     * @param inFileList 
     */
    private static void saveFileList(ArrayList<AlgorithmFileType> inFileList) {
        lock.lock();
        DocumentBuilder builder;
        DocumentBuilderFactory fact;
        File file;
        Document document;
        try {
            file = new File(FileLocations.getAppDataFileLocation() + "AlgorithmList.xml");
            fact = DocumentBuilderFactory.newInstance();
            builder = fact.newDocumentBuilder();
            document = builder.newDocument();
            Element rootElement = document.createElement("AlgorithmList");
            document.appendChild(rootElement);
            for (int i = 0; i < inFileList.size(); i++) {
                Element em1 = document.createElement("Algorithm");
                rootElement.appendChild(em1);
                Element em2 = document.createElement("AlgorithmID");
                em2.setAttribute("ID", inFileList.get(i).getAlgorithmNumber().toString());
                em1.appendChild(em2);
                Element em3 = document.createElement("AlgorithmName");
                em3.appendChild(document.createTextNode(inFileList.get(i).getAlgorithmName()));
                em1.appendChild(em3);
                Element em4 = document.createElement("Description");
                em4.appendChild(document.createTextNode(inFileList.get(i).getDescription()));
                em1.appendChild(em4);
                Element em5 = document.createElement("InputFileType");
                em5.appendChild(document.createTextNode(inFileList.get(i).getInputFileType()));
                em1.appendChild(em5);
                Element em6 = document.createElement("NumberOfParameters");
                em6.setAttribute("Num", inFileList.get(i).getNumberOfParameters().toString());
                em1.appendChild(em6);
                Element em7 = document.createElement("ParameterList");
                for (int j = 0; j < inFileList.get(i).getParameterTypeAndDescription().size(); j++) {
                    Element em8 = document.createElement("ParameterTypeAndDescription");
                    em8.setAttribute("Type", inFileList.get(i).getParameterTypeAndDescription().get(j).get(0));
                    em8.appendChild(document.createTextNode(inFileList.get(i).getParameterTypeAndDescription().get(j).get(1)));
                    em7.appendChild(em8);
                }
                em1.appendChild(em7);
            }
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer;
            try {
                transformer = transformerFactory.newTransformer();
                DOMSource source = new DOMSource(document);
                StreamResult result = new StreamResult(file);
                transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
                transformer.setOutputProperty(OutputKeys.INDENT, "yes");
                transformer.transform(source, result);
            } catch (TransformerException ex) {
                Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(AlgorithmFileManager.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 
     * @return 
     */
    public static ArrayList<AlgorithmFileType> getAlgorithmFileList() {
        ArrayList<AlgorithmFileType> temp = null;
        try {
            temp = algorithmFileList();
        } catch (ParserConfigurationException | SAXException | IOException ex) {
            Logger.getLogger(AlgorithmFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return temp;
    }

    /**
     * 
     * @param inFile
     * @param outFile 
     */
    public static void addFileToList(File inFile, File additionalFolder, AlgorithmFileType inAlgorithmType) {
        ArrayList<AlgorithmFileType> temp = null;
        try {
            temp = algorithmFileList();
            temp.add(inAlgorithmType);
            moveFile(inFile, inAlgorithmType.getFilePath());
            FileUtils.copyDirectory(additionalFolder, new File(FileLocations.getAlgorithmFileLocation() + FilenameUtils.removeExtension(inAlgorithmType.getAlgorithmName()) + "\\" + additionalFolder.getName()));
            saveFileList(temp);
        } catch (ParserConfigurationException | SAXException | IOException ex) {
            Logger.getLogger(AlgorithmFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    /**
     * 
     * @param srcFile
     * @param dstFile 
     */
    private static void moveFile(File srcFile, String dstFile) {
        try {
            File outFile = new File(dstFile);
            FileUtils.copyFile(srcFile, outFile);
        } catch (IOException ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * 
     * @param inFileNumber 
     */
    public static void removeFileFromList(Integer inFileNumber) {
        ArrayList<AlgorithmFileType> temp = getAlgorithmFileList();
        for (int i = 0; i < temp.size(); i++) {
            if (temp.get(i).getAlgorithmNumber() == inFileNumber) {
                try {
                    File deleteFile = new File(temp.get(i).getFolderLocation());
                    FileUtils.forceDelete(deleteFile);
                    temp.remove(i);
                    saveFileList(temp);
                    break;
                } catch (IOException ex) {
                    Logger.getLogger(AlgorithmFileManager.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * 
     * @param inFileNumber
     * @return 
     */
    public static String getAlgorithmFileName(Integer inFileNumber) {
        ArrayList<AlgorithmFileType> temp = getAlgorithmFileList();
        for (AlgorithmFileType tempFile : temp) {
            if (tempFile.getAlgorithmNumber() == inFileNumber) {
                return tempFile.getAlgorithmName();
            }
        }
        return null;
    }

    /**
     * 
     * @param inFileNumber
     * @return 
     */
    public static AlgorithmFileType getAlgorithmFile(Integer inFileNumber) {
        ArrayList<AlgorithmFileType> temp = getAlgorithmFileList();
        for (AlgorithmFileType tempFile : temp) {
            if (tempFile.getAlgorithmNumber() == inFileNumber) {
                return tempFile;
            }
        }
        return null;
    }

}
