package FileManagement.FileTools;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.TransformerException;
import org.w3c.dom.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.apache.commons.io.FileUtils;

/**
 *
 * @author Matty
 */
public class AnalysisFileManager {

    private static DocumentBuilder builder;
    private static DocumentBuilderFactory fact;
    private static File file;
    private static Document document;
    private static Integer fileNumber;
    private static String fileName;
    private static Integer originalFileNumber;
    private static ArrayList<AnalysisFileType> fileList;
    public static Lock lock = new ReentrantLock();

    /**
     * 
     * @param inList 
     */
    private static void setDomInstance(String inList) {
        file = new File(FileLocations.getAppDataFileLocation() + inList);
        fact = DocumentBuilderFactory.newInstance();
        try {
            builder = fact.newDocumentBuilder();
        } catch (Exception ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * 
     */
    private static void analysisFileList() {
        do {
            if (lock.tryLock()) {
                try {
                    setDomInstance("AnalysisList.xml");
                    fileList = new ArrayList<>();
                    if (file.isFile()) {
                        document = builder.parse(file);
                        NodeList nodes_i = document.getDocumentElement().getChildNodes();
                        for (int i = 0; i < nodes_i.getLength(); i++) {
                            Node node_i = nodes_i.item(i);
                            if (node_i.getNodeType() == Node.ELEMENT_NODE && ((Element) node_i).getTagName().equals("AnalysisFile")) {
                                Element fileLocation = (Element) node_i;
                                NodeList nodes_j = fileLocation.getChildNodes();
                                for (int j = 0; j < nodes_j.getLength(); j++) {
                                    Node node_j = nodes_j.item(j);
                                    if (node_j.getNodeType() == Node.ELEMENT_NODE) {
                                        Element outstring = (Element) node_j;
                                        fileNumber = Integer.parseInt(outstring.getAttribute("ID"));
                                        NodeList nodes_k = outstring.getChildNodes();
                                        for (int k = 0; k < nodes_k.getLength(); k++) {
                                            Node node_k = nodes_k.item(k);
                                            if (node_k.getNodeType() == Node.ELEMENT_NODE) {
                                                Element nextstring = (Element) node_k;
                                                if (nextstring.getTagName().equals("OriginalFileNumber")) {
                                                    originalFileNumber = Integer.parseInt(nextstring.getAttribute("Num"));
                                                }
                                                if (nextstring.getTagName().equals("FileName")) {
                                                    fileName = nextstring.getTextContent();
                                                }
                                            }
                                        }
                                        AnalysisFileType temp = new AnalysisFileType(fileNumber, fileName, originalFileNumber);
                                        fileList.add(temp);
                                    }
                                }
                            }
                        }
                    }
                } catch (Exception ex) {
                    Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
                } finally {
                    lock.unlock();
                    break;
                }
                
            }
        } while (true);
    }

    /**
     * 
     * @param inFileList 
     */
    private static void saveFileList(ArrayList<AnalysisFileType> inFileList) {
        lock.lock();
        setDomInstance("AnalysisList.xml");
        document = builder.newDocument();
        Element rootElement = document.createElement("AnalysisList");
        document.appendChild(rootElement);
        for (int i = 0; i < inFileList.size(); i++) {
            Element em1 = document.createElement("AnalysisFile");
            rootElement.appendChild(em1);
            Element em2 = document.createElement("AnalysisID");
            em2.setAttribute("ID", inFileList.get(i).getFileNum().toString());
            em1.appendChild(em2);
            Element em3 = document.createElement("FileName");
            em3.appendChild(document.createTextNode(inFileList.get(i).getFileName()));
            em2.appendChild(em3);
            Element em4 = document.createElement("OriginalFileNumber");
            em4.setAttribute("Num", inFileList.get(i).getOriginalFileNum().toString());
            em2.appendChild(em4);
        }
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer;
        try {
            transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(document);
            StreamResult result = new StreamResult(file);
            transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(source, result);
        } catch (TransformerException ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            lock.unlock();
        }
    }

    protected static void removeFileFromList(Integer inFileNumber) {
        ArrayList<AnalysisFileType> temp = getAnalysisFileList();
        for (int i = 0; i < temp.size(); i++) {
            if (temp.get(i).getFileNum() == inFileNumber) {
                try {
                    File deleteFile = new File(FileLocations.getAnalysisFileLocation() + temp.get(i).getFileName());
                    FileUtils.forceDelete(deleteFile);
                    temp.remove(i);
                    saveFileList(temp);
                    break;
                } catch (IOException ex) {
                    Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * 
     * @return 
     */
    private static ArrayList<AnalysisFileType> getAnalysisFileList() {
        try {
            analysisFileList();
        } catch (Exception ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return fileList;
    }

    /**
     * 
     * @param inFileNumber
     * @return 
     */
    public static AnalysisFileType getAnalysisFile(Integer inFileNumber) {
        ArrayList<AnalysisFileType> temp = getAnalysisFileList();
        for (AnalysisFileType tempFile : temp) {
            if (tempFile.getFileNum() == inFileNumber) {
                return tempFile;
            }
        }
        return null;
    }

    /**
     * 
     * @param inFile 
     */
    public static void addFileToList(AnalysisFileType inFile) {
        try {
            analysisFileList();
            fileList.add(inFile);
            saveFileList(fileList);
        } catch (Exception ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
