package FileManagement.FileTools;

import FileManagement.DataSetTools.CSVTools.CSVTools;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import org.w3c.dom.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.apache.commons.io.FileUtils;

/**
 * 
 * @author Matty
 */
public class DatasetFileManager {

    private static Integer datasetNumber;
    private static String datasetName;
    private static String description;
    private static String fileType;
    private static char delimiter;
    private static Integer analysisFileNumber;
    private static ArrayList<DatasetFileType> fileList;
    private static Lock datasetLock = new ReentrantLock();

    /**
     * 
     */
    private static void datasetFileList() {
        do {
            if (datasetLock.tryLock()) {
                try {
                    File file = new File(FileLocations.getAppDataFileLocation() + "DatasetList.xml");
                    DocumentBuilderFactory fact = DocumentBuilderFactory.newInstance();
                    DocumentBuilder builder = fact.newDocumentBuilder();
                    fileList = new ArrayList<>();
                    if (file.isFile()) {
                        Document document = builder.parse(file);
                        NodeList nodes_i = document.getDocumentElement().getChildNodes();
                        for (int i = 0; i < nodes_i.getLength(); i++) {
                            Node node_i = nodes_i.item(i);
                            if (node_i.getNodeType() == Node.ELEMENT_NODE && ((Element) node_i).getTagName().equals("DataSet")) {
                                Element fileLocation = (Element) node_i;
                                NodeList nodes_j = fileLocation.getChildNodes();
                                for (int j = 0; j < nodes_j.getLength(); j++) {
                                    Node node_j = nodes_j.item(j);
                                    if (node_j.getNodeType() == Node.ELEMENT_NODE) {
                                        Element outstring = (Element) node_j;
                                        datasetNumber = Integer.parseInt(outstring.getAttribute("ID"));
                                        NodeList nodes_k = outstring.getChildNodes();
                                        for (int k = 0; k < nodes_k.getLength(); k++) {
                                            Node node_k = nodes_k.item(k);
                                            if (node_k.getNodeType() == Node.ELEMENT_NODE) {
                                                Element nextstring = (Element) node_k;
                                                if (nextstring.getTagName().equals("AnalysisFileNumber")) {
                                                    analysisFileNumber = Integer.parseInt(nextstring.getAttribute("Num"));
                                                }
                                                if (nextstring.getTagName().equals("DatasetName")) {
                                                    datasetName = nextstring.getTextContent();
                                                }
                                                if (nextstring.getTagName().equals("FileType")) {
                                                    fileType = nextstring.getTextContent();
                                                }
                                                if (nextstring.getTagName().equals("Description")) {
                                                    description = nextstring.getTextContent();
                                                }
                                                if (nextstring.getTagName().equals("Delimiter")) {
                                                    delimiter = nextstring.getTextContent().charAt(0);
                                                }
                                            }
                                        }
                                        DatasetFileType temp = new DatasetFileType(datasetNumber, datasetName, delimiter, description, fileType, analysisFileNumber);
                                        fileList.add(temp);
                                    }
                                }
                            }
                        }
                    }
                } catch (Exception ex) {
                    Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
                } finally {
                    datasetLock.unlock();
                    break;
                }
            }
        } while (true);
    }

    /**
     * 
     * @param inFileList 
     */
    private static void saveFileList(ArrayList<DatasetFileType> inFileList) {
        datasetLock.lock();
        try {
            File file = new File(FileLocations.getAppDataFileLocation() + "DatasetList.xml");
            DocumentBuilderFactory fact = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = fact.newDocumentBuilder();
            Document document = builder.newDocument();
            Element rootElement = document.createElement("DatasetList");
            document.appendChild(rootElement);
            for (int i = 0; i < inFileList.size(); i++) {
                Element datasetElement = document.createElement("DataSet");
                rootElement.appendChild(datasetElement);
                Element iDElement = document.createElement("DataSetID");
                iDElement.setAttribute("ID", inFileList.get(i).getFileNum().toString());
                datasetElement.appendChild(iDElement);
                Element nameElement = document.createElement("DatasetName");
                nameElement.appendChild(document.createTextNode(inFileList.get(i).getFileName()));
                iDElement.appendChild(nameElement);
                Element delimiterElement = document.createElement("Delimiter");
                delimiterElement.appendChild(document.createTextNode(Character.toString(inFileList.get(i).getDelimiter())));
                iDElement.appendChild(delimiterElement);
                Element descriptionElement = document.createElement("Description");
                descriptionElement.appendChild(document.createTextNode(inFileList.get(i).getDescription()));
                iDElement.appendChild(descriptionElement);
                Element typeElement = document.createElement("FileType");
                typeElement.appendChild(document.createTextNode(inFileList.get(i).getFileType()));
                iDElement.appendChild(typeElement);
                Element analysisElement = document.createElement("AnalysisFileNumber");
                analysisElement.setAttribute("Num", inFileList.get(i).getAnalysisFileNumber().toString());
                iDElement.appendChild(analysisElement);
            }
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer;
            try {
                transformer = transformerFactory.newTransformer();
                DOMSource source = new DOMSource(document);
                StreamResult result = new StreamResult(file);
                transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
                transformer.setOutputProperty(OutputKeys.INDENT, "yes");
                transformer.transform(source, result);


            } catch (TransformerException ex) {
                Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (ParserConfigurationException ex) {
        } finally {
            datasetLock.unlock();
        }
    }

    /**
     * 
     * @return 
     */
    public static ArrayList<DatasetFileType> getDatasetFileList() {
        try {
            datasetFileList();


        } catch (Exception ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return fileList;
    }

    /**
     * 
     * @param inFile
     * @param outFile 
     */
    public static void addFileToList(File inFile, DatasetFileType inDatasetType) throws FileExistsException {
        if (!checkIfFileInList(inDatasetType.getFileName())){
            boolean moved = moveFile(inFile, FileLocations.getDatasetFileLocation() + inDatasetType.getFileName());
            if (moved) {
                datasetFileList();
                fileList.add(inDatasetType);
                saveFileList(fileList);
            }
        } else {
            throw new FileExistsException();
        }
    }

    private static boolean checkIfAlreadyAdded(File inFile) {
        if (inFile.exists()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 
     * @param filename
     * @param delimiter
     * @param inArray
     * @param outFile 
     */
    public static void addArrayToList(String filename, char delimiter, ArrayList<ArrayList<String>> inArray, DatasetFileType inFileType) throws FileExistsException {
        if (!checkIfFileInList(filename)) {
            datasetFileList();
            fileList.add(inFileType);
            CSVTools.saveArrayToCSV(filename, delimiter, inArray);
            saveFileList(fileList);
        } else {
            throw new FileExistsException();
        }
    }

    /**
     * 
     * @param srcFile
     * @param dstFile 
     */
    private static boolean moveFile(File srcFile, String dstFile) {
        try {
            File outFile = new File(dstFile);
            FileUtils.copyFile(srcFile, outFile);
            return true;




        } catch (IOException ex) {
            Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return false;
    }

    /**
     * 
     * @param inFileNumber 
     */
    public static void removeFileFromList(Integer inFileNumber) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (int i = 0; i < temp.size(); i++) {
            if (temp.get(i).getFileNum() == inFileNumber) {
                try {
                    File deleteFile = new File(FileLocations.getDatasetFileLocation() + temp.get(i).getFileName());
                    FileUtils.forceDelete(deleteFile);
                    if (temp.get(i).getAnalysisFileNumber() != -1) {
                        AnalysisFileManager.removeFileFromList(temp.get(i).getAnalysisFileNumber());
                    }
                    temp.remove(i);
                    saveFileList(temp);
                    break;


                } catch (IOException ex) {
                    Logger.getLogger(DatasetFileManager.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * 
     * @param inFileNumber
     * @return 
     */
    public static String getDatasetFileName(Integer inFileNumber) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (DatasetFileType tempFile : temp) {
            if (tempFile.getFileNum() == inFileNumber) {
                return tempFile.getFileName();
            }
        }
        return null;
    }

    /**
     * 
     * @param inFileNumber
     * @return 
     */
    public static DatasetFileType getFile(Integer inFileNumber) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (DatasetFileType tempFile : temp) {
            if (tempFile.getFileNum() == inFileNumber) {
                return tempFile;
            }
        }
        return null;
    }

    /**
     * 
     * @param inFileName
     * @return 
     */
    public static DatasetFileType getDataset(String inFileName) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (DatasetFileType tempFile : temp) {
            if (tempFile.getFileName().equals(inFileName)) {
                return tempFile;
            }
        }
        return null;
    }

    /**
     * 
     * @param inType 
     */
    public static void updateFile(DatasetFileType inType) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (int i = 0; i < temp.size(); i++) {
            if (temp.get(i).getFileNum() == inType.getFileNum()) {
                temp.set(i, inType);
                saveFileList(temp);
                break;
            }
        }
    }

    /**
     * 
     * @return 
     */
    public static String[] getFileNameAsArray() {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        String[] tempArray = new String[temp.size() + 1];
        tempArray[0] = "";
        for (int i = 0; i < temp.size(); i++) {
            String string = temp.get(i).getFileName();
            tempArray[i + 1] = string;
        }
        return tempArray;
    }

    private static boolean checkIfFileInList(String inFilename) {
        ArrayList<DatasetFileType> temp = getDatasetFileList();
        for (DatasetFileType tempFile : temp) {
            if (tempFile.getFileName() == inFilename) {
                return true;
            }
        }
        return false;
    }
}
