package GUI.TestToolGUI.AlgorithmRunPane;

import AlgorithmTools.RunAlgorithm;
import FileManagement.FileTools.AlgorithmFileType;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.FileCounter;
import FileManagement.FileTools.ResultFileType;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import net.java.dev.designgridlayout.DesignGridLayout;
import org.apache.commons.io.FilenameUtils;

/**
 *
 * @author Matty
 */
public class RunAlgorithmPane extends JFrame implements ActionListener, Runnable {

    private JTextArea outputPane;
    private JPanel paramPanel;
    private JTextField[] textEntry;
    private JLabel[] labels;
    private ArrayList<ArrayList<String>> parameterDetails;
    private JScrollPane outputScrollPane, paramScrollPane;
    private AlgorithmFileType algorithm;
    private DesignGridLayout layout;
    private JComboBox datasetListBox;
    private JButton runButton;
    private DatasetFileType inputDataset;
    private ResultFileType resultFile;
    private boolean validEntry;
    private OutputStream os;
    private InputStream is;

    /**
     * 
     * @param inAlgorithm 
     */
    public RunAlgorithmPane(AlgorithmFileType inAlgorithm) {
        super();
        this.setTitle("Run Algorithm - " + inAlgorithm.getAlgorithmName());
        algorithm = inAlgorithm;
        parameterDetails = algorithm.getParameterTypeAndDescription();
    }

    /**
     * 
     */
    private void createAndShow() {
        layout = new DesignGridLayout(this);
        datasetListBox = new JComboBox(DatasetFileManager.getFileNameAsArray());
        datasetListBox.addActionListener(this);

        outputPane = new JTextArea();
        outputPane.setEditable(false);
        outputPane.setPreferredSize(new Dimension(RunAlgorithmPane.WIDTH, 300));
        outputScrollPane = new JScrollPane(outputPane);

        paramPanel = new JPanel();
        paramPanel.setLayout(new GridLayout(0, 2));
        paramScrollPane = new JScrollPane(paramPanel);
        textEntry = new JTextField[algorithm.getNumberOfParameters()];
        labels = new JLabel[algorithm.getNumberOfParameters()];
        setEntryBoxes();

        runButton = new JButton("Run");
        runButton.addActionListener(this);
        redirectSystemStreams();


        layout.row().grid().add(new JLabel("Choose Dataset")).add(datasetListBox);
        layout.row().grid().add(new JLabel("Set Parameters"));
        layout.row().grid().add(paramScrollPane);
        layout.row().grid().add(new JLabel("Output:"));
        layout.row().grid().add(outputScrollPane);
        layout.row().grid().add(runButton);
    }

    /**
     * 
     */
    private void setEntryBoxes() {
        for (int i = 0; i < textEntry.length; i++) {
            labels[i] = new JLabel("Param " + (i + 1));
            textEntry[i] = new JTextField();
            textEntry[i].setToolTipText(getParameterDetails(i));
            paramPanel.add(labels[i]);
            paramPanel.add(textEntry[i]);
            if (i == 0) {
                textEntry[i].setEditable(false);
            }
        }
    }

    private void updateTextArea(final String text) {
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                outputPane.append(text);
            }
        });
    }

    private void redirectSystemStreams() {
        OutputStream out = new OutputStream() {

            @Override
            public void write(int b) throws IOException {
                updateTextArea(String.valueOf((char) b));
            }

            @Override
            public void write(byte[] b, int off, int len) throws IOException {
                updateTextArea(new String(b, off, len));
            }

            @Override
            public void write(byte[] b) throws IOException {
                write(b, 0, b.length);
            }
        };

        System.setOut(new PrintStream(out, true));
        System.setErr(new PrintStream(out, true));
    }

    /**
     * 
     * @param inString 
     */
    public void setOutput(String inString) {
        outputPane.append(inString);
        outputPane.append("\n");
    }

    /**
     * 
     * @param i
     * @return 
     */
    private String getParameterDetails(int i) {
        StringBuilder string = new StringBuilder();
        string.append("Parameter Type :");
        string.append(parameterDetails.get(i).get(0));
        string.append(" Description : ");
        string.append(parameterDetails.get(i).get(1));
        return string.toString();
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }

    /**
     * 
     * @param inFileName 
     */
    public void setInputParameterTextField(String inFileName) {
        try {
            textEntry[0].setText(inFileName);
            textEntry[1].setText(FilenameUtils.removeExtension(inFileName) + "Result[" + FileCounter.getCurrentFileNumber() + "]." + FilenameUtils.getExtension(inFileName));
        } catch (IOException ex) {
        }
    }

    /**
     * 
     * @return 
     */
    public String getResultFileName() {
        return textEntry[1].getText();
    }

    /**
     * 
     * @return 
     */
    public String[] getParamValues() {
        String[] temp = new String[algorithm.getNumberOfParameters()];
        temp[0] = inputDataset.getFilePath();
        temp[1] = resultFile.getFilePath();
        for (int i = 2; i < textEntry.length; i++) {
            String temp1 = new String();
            temp1 = textEntry[i].getText();
        }
        return temp;
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == datasetListBox && datasetListBox.getSelectedIndex() > 0) {
            inputDataset = DatasetFileManager.getDataset(datasetListBox.getSelectedItem().toString());
            setInputParameterTextField(inputDataset.getFileName());
        }
        if (ae.getSource() == runButton) {
            try {
                System.out.println("Running....");
                resultFile = new ResultFileType(inputDataset.getFileNum(), getResultFileName(), inputDataset.getDelimiter(), null);
                String description = "Result file from " + algorithm.getAlgorithmName() + "with parameters:-" + getParamValues();
                resultFile.setDescription(description);
                RunAlgorithm runningThread = new RunAlgorithm(getParamValues(), algorithm, resultFile);
                SwingUtilities.invokeLater(runningThread);
            } catch (IOException ex) {
                Logger.getLogger(RunAlgorithmPane.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                System.out.println("Finished");
            }
        }
    }
}
