package GUI.TestToolGUI.CSVEditor;

import FileManagement.DataSetTools.CSVTools.CSVTools;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.ResultFileManager;
import FileManagement.FileTools.ResultFileType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import quick.dbtable.DBTable;

/**
 *
 * @author Matty
 */
public class CSVEditor extends JFrame implements ActionListener, Runnable {

    private JComboBox comboBox;
    private DBTable dBTable1;
    private ArrayList<ArrayList<String>> tempArray;
    private JPanel panel;
    private JButton removeColumnButton, saveDatasetButton;
    private int selectedColumn;
    protected DatasetFileType datasetFileShowing;
    protected ResultFileType resultFileShowing;
    public static final String DATASET = "Dataset";
    public static final String RESULTFILE = "ResultFile";
    private SaveDatasetPanel savePanel;
    protected String typeOfFile;
    private boolean editable, transactional;

    /**
     * 
     * @param fileNumber
     * @param inEditable
     * @param inFileType 
     */
    public CSVEditor(Integer fileNumber, boolean inEditable, final String inFileType) {
        this.setTitle("CSV Editor");
        typeOfFile = inFileType;
        editable = inEditable;
        transactional = false;
        if (typeOfFile.equals(DATASET)) {
            datasetFileShowing = DatasetFileManager.getFile(fileNumber);
            if (datasetFileShowing.getFileType().equals(DatasetFileType.RELATIONAL)) {
                tempArray = CSVTools.getCSVAsArray(datasetFileShowing.getFilePath(), datasetFileShowing.getDelimiter());
            }
            if (datasetFileShowing.getFileType().equals(DatasetFileType.TRANSACTIONAL)) {
                tempArray = CSVTools.getSquareArray(CSVTools.getCSVAsArray(datasetFileShowing.getFilePath(), datasetFileShowing.getDelimiter()));
                transactional = true;
            }
        }
        if (typeOfFile.equals(RESULTFILE)) {
            resultFileShowing = ResultFileManager.getFile(fileNumber);
            if (DatasetFileManager.getFile(resultFileShowing.getOriginalDatasetNumber()).getFileType().equals(DatasetFileType.RELATIONAL)) {
                tempArray = CSVTools.getCSVAsArray(resultFileShowing.getFilePath(), resultFileShowing.getDelimiter());
            }
            if (DatasetFileManager.getFile(resultFileShowing.getOriginalDatasetNumber()).getFileType().equals(DatasetFileType.TRANSACTIONAL)) {
                tempArray = CSVTools.getSquareArray(CSVTools.getCSVAsArray(resultFileShowing.getFilePath(), resultFileShowing.getDelimiter()));
                transactional = true;
            }
        }
    }

    /**
     * 
     * @param inArray 
     */
    private void loadArrayAndDisplay(ArrayList<ArrayList<String>> inArray) {
        try {
            Vector<Vector<String>> array = changeArrayListToVector(inArray);
            dBTable1 = new DBTable();
            dBTable1.refresh(array);

            this.setSize(600, 500);
            if (editable) {
                setControlPanel();
            }
            add(dBTable1);
            setVisible(true);
        } catch (Exception ex) {
            Logger.getLogger(CSVEditor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * 
     */
    private void setControlPanel() {
        dBTable1.createControlPanel();
        panel = dBTable1.getControlPanel();
        if (!transactional) {
            comboBox = new JComboBox(getColumnList());
            comboBox.setSelectedIndex(0);
            selectedColumn = 0;
            removeColumnButton = new JButton("remove column");
            removeColumnButton.addActionListener((ActionListener) this);
            panel.add(new JLabel());
            panel.add(comboBox);
            panel.add(removeColumnButton);
            comboBox.addActionListener((ActionListener) this);
        }
        saveDatasetButton = new JButton("Save");
        saveDatasetButton.addActionListener(this);
        panel.add(saveDatasetButton);
    }

    /**
     * 
     * @param inArray
     * @return 
     */
    private Vector<Vector<String>> changeArrayListToVector(ArrayList<ArrayList<String>> inArray) {
        Vector<Vector<String>> array = new Vector<Vector<String>>();
        for (ArrayList<String> temp : inArray) {
            Vector<String> temp2 = new Vector<String>(temp);
            array.add(temp2);
        }
        return array;
    }

    /**
     * 
     * @param inVector
     * @return 
     */
    private ArrayList<ArrayList<String>> changeVectorToArrayList(Vector<Vector<String>> inVector) {
        ArrayList<ArrayList<String>> temp = new ArrayList<>();
        for (Vector<String> vector : inVector) {
            ArrayList<String> tempArray = new ArrayList<>(vector);
            temp.add(tempArray);
        }
        return temp;
    }

    /**
     * 
     * @return 
     */
    private String[] getColumnList() {
        String[] columnString = new String[dBTable1.getColumnCount()];
        for (int i = 0; i < dBTable1.getColumnCount(); i++) {
            String inString = dBTable1.getColumn(i).getColumnName();
            columnString[i] = inString;
        }
        return columnString;
    }

    /**
     * 
     * @return 
     */
    protected ArrayList<ArrayList<String>> getTableData() {
        ArrayList<ArrayList<String>> temp = changeVectorToArrayList(dBTable1.getDataVector());
        if (savePanel.fileType.equals(DatasetFileType.TRANSACTIONAL)) {
            CSVTools.removeSquareArray(temp);
        }
        return temp;
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == comboBox) {
            selectedColumn = comboBox.getSelectedIndex();
        }
        if (ae.getSource() == removeColumnButton) {
            dBTable1.removeColumn(dBTable1.getColumn(selectedColumn));
            comboBox.removeItemAt(selectedColumn);
        }
        if (ae.getSource() == saveDatasetButton) {
            savePanel = new SaveDatasetPanel(this);
            savePanel.run();
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        if (!tempArray.isEmpty()) {
            loadArrayAndDisplay(tempArray);
        }
    }
}
