package GUI.TestToolGUI.CSVEditor;

import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.FileCounter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import net.java.dev.designgridlayout.DesignGridLayout;
import org.apache.commons.io.FilenameUtils;

/**
 *
 * @author Matty
 */
public class SaveDatasetPanel extends JFrame implements ActionListener, Runnable {

    private JPanel panel;
    private JLabel fileName, fileDescription;
    private JTextField inFileName;
    private JTextArea description;
    private JScrollPane scrollPane;
    private JButton addFile, cancel;
    private JComboBox fileTypeList;
    protected String fileType;
    private Boolean validEntry;
    private DesignGridLayout layout;
    private File file;
    private static final String[] fileTypes = {DatasetFileType.RELATIONAL, DatasetFileType.TRANSACTIONAL};
    private final char[] delimiters = {',', ' '};
    private final String[] delimitersAsString = {"comma", "space"};
    private char delimiter;
    private JComboBox delimiterComboBox;
    CSVEditor parentPane;

    /**
     * 
     * @param inParentPane 
     */
    public SaveDatasetPanel(CSVEditor inParentPane) {
        parentPane = inParentPane;
    }

    /**
     * 
     */
    private void createAndShow() {
        setTitle("Save Dataset");
        validEntry = true;
        panel = new JPanel();
        layout = new DesignGridLayout(panel);
        addEntryBoxes();
        this.add(panel);
    }

    /**
     * 
     */
    private void addEntryBoxes() {
        fileName = new JLabel("File Name");
        inFileName = new JTextField(20);
        inFileName.setEditable(true);
        StringBuilder filename = new StringBuilder();
        filename.append(FilenameUtils.removeExtension(parentPane.datasetFileShowing.getFileName()));
        filename.append("[");
        try {
            filename.append(FileCounter.getCurrentFileNumber().toString());
        } catch (IOException ex) {
            Logger.getLogger(SaveDatasetPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
        filename.append("].");
        filename.append(FilenameUtils.getExtension(parentPane.datasetFileShowing.getFileName()));
        inFileName.setText(filename.toString());

        fileDescription = new JLabel("Description");
        description = new JTextArea(10, 10);
        description.setLineWrap(true);
        scrollPane = new JScrollPane(description);
        if (parentPane.typeOfFile.equals(CSVEditor.DATASET)) {
            description.setText(parentPane.datasetFileShowing.getDescription());
        }
        if (parentPane.typeOfFile.equals(CSVEditor.RESULTFILE)) {
            description.setText(parentPane.resultFileShowing.getDescription());
        }

        fileTypeList = new JComboBox(fileTypes);
        if (parentPane.typeOfFile.equals(CSVEditor.DATASET)) {
            fileTypeList.setSelectedItem(parentPane.datasetFileShowing.getFileType());
        }
        if (parentPane.typeOfFile.equals(CSVEditor.RESULTFILE)) {
            fileTypeList.setSelectedItem(parentPane.resultFileShowing.getOriginalFileType());
        }
        fileType = (String) fileTypeList.getSelectedItem();
        fileTypeList.addActionListener(this);

        delimiterComboBox = new JComboBox(delimitersAsString);
        delimiterComboBox.addActionListener(this);
        delimiterComboBox.setSelectedIndex(0);
        delimiter = delimiters[0];

        addFile = new JButton("Add");
        cancel = new JButton("Cancel");
        addFile.addActionListener((ActionListener) this);
        cancel.addActionListener((ActionListener) this);

        layout.row().grid().add(fileName).add(inFileName);
        layout.row().grid().add(new JLabel("Delimiter")).add(delimiterComboBox);
        layout.row().grid().add(fileDescription).add(scrollPane);
        layout.row().grid().add(new JLabel("Dataset Type")).add(fileTypeList);
        layout.row().grid().add(addFile).add(cancel);
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == addFile && validEntry) {
            try {
                DatasetFileType newFile = new DatasetFileType(inFileName.getText(), delimiter, description.getText(), fileType);
                DatasetFileManager.addArrayToList(newFile.getFilePath(), newFile.getDelimiter(), parentPane.getTableData(), newFile);
                description.setText("");
                inFileName.setText("");
                validEntry = false;
            } catch (Exception ex) {
                Logger.getLogger(SaveDatasetPanel.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        if (ae.getSource() == fileTypeList) {
            fileType = (String) fileTypeList.getSelectedItem();
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }
}
