package GUI.TestToolGUI.ComparisonPane;

import MeasurementTools.MeasurementInfoTools.CalculateComparisonMeasures;
import MeasurementTools.MeasurementInfoTools.ListOfMeasures;
import FileManagement.DataSetTools.CSVTools.CSVTools;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.ResultFileManager;
import FileManagement.FileTools.ResultFileType;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.concurrent.CountDownLatch;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import net.java.dev.designgridlayout.DesignGridLayout;

/**
 *
 * @author Matty
 */
public class ComparisonPane extends JFrame implements ActionListener, Runnable {

    private JFrame parentPane;
    private DesignGridLayout layout;
    private JComboBox selectDatasetBox, selectResultBox1, selectResultBox2;
    private ArrayList<JComboBox> selectResultSet;
    private ArrayList<DatasetFileType> fileList;
    private ArrayList<ResultFileType> resultFileList;
    private JList resultList1, resultList2, selectedResultList;
    private int originalDatasetNumber;
    private JScrollPane scrollPane, measureScrollPane;
    private JPanel QIDPanel, measurePanel;
    private DefaultComboBoxModel resultList1Model, resultList2Model;
    private String[] temp1, temp2;
    private int resultFile1, resultFile2;
    private JLabel QIDChoice, measureChoice;
    private JCheckBox[] QIDCheckBox, measureCheckBox;
    private JButton runButton;
    private int selectedFileNumber;
    private ArrayList<String> listOfMeasures;
    private JLabel infoLabel;

    /**
     * 
     * @param inParentPane 
     */
    public ComparisonPane(JFrame inParentPane) {
        super();
        this.setTitle("Comparison Tool");
        parentPane = inParentPane;
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                parentPane.setVisible(true);
            }
        });
    }

    /**
     * 
     * @param noOfColumns 
     */
    private void setQIDPanel(int noOfColumns) {
        QIDPanel.removeAll();
        QIDPanel.setLayout(new GridLayout(0, 4));
        QIDCheckBox = new JCheckBox[noOfColumns];
        for (int i = 0; i < noOfColumns; i++) {
            JCheckBox temp = new JCheckBox();
            QIDCheckBox[i] = temp;
            QIDPanel.add(new JLabel(" column " + (i + 1)));
            QIDPanel.add(QIDCheckBox[i]);
        }
        QIDPanel.validate();
        this.pack();
        this.validate();
    }

    /**
     * 
     * @param inType 
     */
    private void setAvailableMeasures(String inType) {
        listOfMeasures = ListOfMeasures.getGeneralMeasures();
        if (inType.equals(DatasetFileType.RELATIONAL)) {
            ArrayList<String> temp = ListOfMeasures.getRelationalMeasures();
            listOfMeasures.addAll(temp);
        }
        if (inType.equals(DatasetFileType.TRANSACTIONAL)) {
            ArrayList<String> temp = ListOfMeasures.getTransactionalMeasures();
            listOfMeasures.addAll(temp);
        }
        measurePanel.removeAll();
        measurePanel.setLayout(new GridLayout(0, 4));
        measureCheckBox = new JCheckBox[listOfMeasures.size()];
        for (int i = 0; i < listOfMeasures.size(); i++) {
            JCheckBox temp = new JCheckBox();
            measureCheckBox[i] = temp;
            measurePanel.add(new JLabel(listOfMeasures.get(i)));
            measurePanel.add(measureCheckBox[i]);
        }
        measurePanel.validate();
        this.pack();
        this.validate();
    }

    /**
     * 
     */
    private void initialize() {
        layout = new DesignGridLayout(this);
        fileList = DatasetFileManager.getDatasetFileList();
        resultFileList = ResultFileManager.getResultFileList();

        JLabel datasetChoice = new JLabel("Select Dataset");
        selectDatasetBox = new JComboBox(DatasetFileManager.getFileNameAsArray());
        selectDatasetBox.addActionListener(this);

        JLabel resultChoice1 = new JLabel("Select Result File 1 To Compare");
        selectResultBox1 = new JComboBox();
        selectResultBox1.setEnabled(false);
        selectResultBox1.addActionListener(this);

        JLabel resultChoice2 = new JLabel("Select Result File 2 To Compare");
        selectResultBox2 = new JComboBox();
        selectResultBox2.setEnabled(false);
        selectResultBox2.addActionListener(this);

        QIDChoice = new JLabel("Select QID's");
        QIDPanel = new JPanel();
        scrollPane = new JScrollPane(QIDPanel);
        QIDPanel.setSize(100, ComparisonPane.WIDTH);
        QIDChoice.setVisible(false);

        measureChoice = new JLabel("Select Measures");
        measurePanel = new JPanel();
        measureScrollPane = new JScrollPane(measurePanel);
        measurePanel.setSize(100, ComparisonPane.WIDTH);

        runButton = new JButton("RUN");
        runButton.addActionListener(this);

        infoLabel = new JLabel("Select Files");

        layout.row().grid().add(datasetChoice).add(selectDatasetBox);
        layout.row().grid().add(resultChoice1).add(selectResultBox1);
        layout.row().grid().add(resultChoice2).add(selectResultBox2);
        layout.row().grid().add(QIDChoice);
        layout.row().grid().add(scrollPane);
        layout.row().grid().add(measureChoice);
        layout.row().grid().add(measureScrollPane);
        layout.row().grid().add(runButton);
        layout.row().grid().add(infoLabel);
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if ((ae.getSource() == selectDatasetBox) && (selectDatasetBox.getSelectedIndex() > 0)) {
            originalDatasetNumber = fileList.get(selectDatasetBox.getSelectedIndex() - 1).getFileNum();
            temp1 = ResultFileManager.getFilesFromOriginalDatasetAsArray(originalDatasetNumber);
            resultList1Model = new DefaultComboBoxModel(temp1);
            resultList1 = new JList(resultList1Model);
            selectResultBox1.setModel(resultList1Model);
            selectResultBox1.setEnabled(true);
            selectResultBox1.validate();
            selectedFileNumber = selectDatasetBox.getSelectedIndex() - 1;

            if (fileList.get(selectedFileNumber).getFileType().equals(DatasetFileType.RELATIONAL)) {
                setAvailableMeasures(DatasetFileType.RELATIONAL);
                QIDChoice.setVisible(true);
                ArrayList<ArrayList<String>> tempArray = CSVTools.getCSVAsArray(fileList.get(selectedFileNumber).getFilePath(), fileList.get(selectedFileNumber).getDelimiter());
                setQIDPanel(tempArray.get(0).size());
            }
            if (fileList.get(selectedFileNumber).getFileType().equals(DatasetFileType.TRANSACTIONAL)) {
                setAvailableMeasures(DatasetFileType.TRANSACTIONAL);
                QIDChoice.setVisible(false);
                QIDPanel.removeAll();
                this.pack();
                QIDPanel.validate();
                scrollPane.validate();
                this.validate();
            }
        }
        if ((ae.getSource() == selectResultBox1) && (selectDatasetBox.getSelectedIndex() > 0) && (selectResultBox1.getSelectedIndex() > 0)) {

            temp2 = ResultFileManager.getFilesFromOriginalDatasetAsArray(originalDatasetNumber);
            temp2[selectResultBox1.getSelectedIndex()] = "";
            resultList2Model = new DefaultComboBoxModel(temp2);
            resultList2 = new JList(resultList2Model);
            selectResultBox2.setModel(resultList2Model);
            selectResultBox2.setEnabled(true);
            selectResultBox2.validate();
            resultFile1 = resultFileList.get(selectResultBox1.getSelectedIndex() - 1).getFileNum();
        }
        if ((ae.getSource() == selectResultBox2) && (selectResultBox2.getSelectedIndex() > 0) && (selectResultBox2.getSelectedIndex() != selectResultBox1.getSelectedIndex())) {

            resultFile2 = resultFileList.get(selectResultBox2.getSelectedIndex() - 1).getFileNum();
        }
        if (ae.getSource() == runButton && validEntry()) {
            if (fileList.get(selectedFileNumber).getFileType().equals(DatasetFileType.RELATIONAL)) {
                int[] QIDs = getQIDsSelected();
                ArrayList<String> measures = getMeasureList();
                // create a new countdown latch
                CountDownLatch waitForThreadLatch = new CountDownLatch(2);
                // construct a new comparison measure thread for result-dataset 1
                CalculateComparisonMeasures calculate = new CalculateComparisonMeasures(resultFile1, originalDatasetNumber, measures, QIDs, waitForThreadLatch);
                // construct a new comparison measure thread for result-dataset 2
                CalculateComparisonMeasures calculate2 = new CalculateComparisonMeasures(resultFile2, originalDatasetNumber, measures, QIDs, waitForThreadLatch);
                // construct a new display pane thread
                DisplayComparisonResults display = new DisplayComparisonResults(originalDatasetNumber, resultFile1, resultFile2, measures, QIDs);
                try {
                    // run comparison thread 1
                    calculate.run();
                    infoLabel.setText("Running first comparison....");
                    this.validate();
                    // run comparison thread 2
                    calculate2.run();
                    infoLabel.setText("Running second comparison...");
                    this.validate();
                    // wait for those threads to countdown the latch to zero
                    waitForThreadLatch.await();
                    // then display results
                    SwingUtilities.invokeLater(display);
                    infoLabel.setText("Diplaying results....");
                    this.validate();
                    infoLabel.setText("Run Another Comaprison?");
                    this.validate();
                } catch (InterruptedException ex) {
                }
            }
        }
    }

    /**
     * 
     * @return 
     */
    private int[] getQIDsSelected() {
        ArrayList<Integer> tempArray = new ArrayList<>();
        for (int i = 0; i < QIDCheckBox.length; i++) {
            if (QIDCheckBox[i].isSelected()) {
                tempArray.add(i);
            }
        }
        int[] ret = new int[tempArray.size()];
        Iterator<Integer> iterator = tempArray.iterator();
        for (int i = 0; i < ret.length; i++) {
            ret[i] = iterator.next().intValue();
        }

        return ret;
    }

    /**
     * 
     * @return 
     */
    private ArrayList<String> getMeasureList() {
        ArrayList<String> temp = new ArrayList<>();
        for (int i = 0; i < measureCheckBox.length; i++) {
            if (measureCheckBox[i].isSelected()) {
                temp.add(listOfMeasures.get(i));
            }
        }
        return temp;
    }

    /**
     * 
     * @return 
     */
    private boolean validEntry() {
        if (selectDatasetBox.getSelectedIndex() > 0) {
            if (selectResultBox1.getSelectedIndex() > 0) {
                if ((selectResultBox2.getSelectedIndex() > 0) && (selectResultBox1.getSelectedIndex() != selectResultBox2.getSelectedIndex())) {
                    if (!new ArrayList<>(getMeasureList()).isEmpty()) {
                        int[] temp = getQIDsSelected();
                        if (!(temp == null)) {
                            return true;
                        }
                    }
                }

            }
        }
        infoLabel.setText("Missing Values! please fill out form");
        return false;
    }

    /**
     * 
     */
    @Override
    public void run() {
        initialize();
        this.pack();
        this.setVisible(true);
    }
}
