package GUI.TestToolGUI.ComparisonPane;

import MeasurementTools.MeasurementInfoTools.MeasurementInformation;
import FileManagement.FileTools.AnalysisFileManager;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.ResultFileManager;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.html.simpleparser.HTMLWorker;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *
 * @author Matty
 */
public class DisplayComparisonResults extends JFrame implements Runnable, ActionListener {

    private JPanel infoPanel;
    private JScrollPane scrollPane;
    private JButton showGraphButton, saveToPDFButton;
    private int originalFile, resultFile1, resultFile2;
    private ArrayList<String> measures;
    private int[] QIDs;
    private MeasurementInformation originalInfo, result1Info, result2Info;
    private StringBuffer infoHTMLString;

    /**
     * 
     * @param inOriginalFile
     * @param inResultFile1
     * @param inResultFile2
     * @param inMeasures
     * @param inQIDs 
     */
    public DisplayComparisonResults(int inOriginalFile, int inResultFile1, int inResultFile2, ArrayList<String> inMeasures, int[] inQIDs) {
        originalFile = inOriginalFile;
        resultFile1 = inResultFile1;
        resultFile2 = inResultFile2;
        measures = inMeasures;
        QIDs = inQIDs;
    }

    /**
     * 
     */
    private void createAndShow() {
        this.setLayout(new BorderLayout());

        this.setSize(200, 300);
        this.setTitle("Comparison Results");
        infoPanel = new JPanel();
        infoPanel.setSize(400, 400);
        infoPanel.setLayout(new GridLayout(0, 4));
        scrollPane = new JScrollPane(infoPanel);
        add(scrollPane, BorderLayout.CENTER);
        infoPanel.setBackground(Color.WHITE);
        JPanel buttonHolder = new JPanel();
        buttonHolder.setLayout(new GridLayout(1, 4));

        showGraphButton = new JButton("Show Graph");
        showGraphButton.addActionListener(this);
        buttonHolder.add(showGraphButton);

        saveToPDFButton = new JButton("Save To PDF");
        saveToPDFButton.addActionListener(this);
        buttonHolder.add(saveToPDFButton);

        add(buttonHolder, BorderLayout.SOUTH);

        infoHTMLString = new StringBuffer();
        setValues();
    }

    /**
     * 
     */
    private void setValues() {
        originalInfo = new MeasurementInformation(QIDs);
        result1Info = new MeasurementInformation(QIDs);
        result2Info = new MeasurementInformation(QIDs);
        try {
            originalInfo.readMeasures(AnalysisFileManager.getAnalysisFile(DatasetFileManager.getFile(originalFile).getAnalysisFileNumber()).getFilePath());
            result1Info.readMeasures(AnalysisFileManager.getAnalysisFile(ResultFileManager.getFile(resultFile1).getAnalysisFileNumber()).getFilePath());
            result2Info.readMeasures(AnalysisFileManager.getAnalysisFile(ResultFileManager.getFile(resultFile2).getAnalysisFileNumber()).getFilePath());
        } catch (IOException ex) {
        }
        HashMap<String, Float> originalMeasurelist = originalInfo.getMeasurelist();
        HashMap<String, Float> result1Measurelist = result1Info.getMeasurelist();
        HashMap<String, Float> result2Measurelist = result2Info.getMeasurelist();

        JLabel tempLabel1Heading = new JLabel(DatasetFileManager.getFile(originalFile).getFileName());
        tempLabel1Heading.setBorder(BorderFactory.createLineBorder(Color.black));
        JLabel tempLabel2Heading = new JLabel(ResultFileManager.getFile(resultFile1).getFileName());
        tempLabel2Heading.setBorder(BorderFactory.createLineBorder(Color.black));
        JLabel tempLabel3Heading = new JLabel(ResultFileManager.getFile(resultFile2).getFileName());
        tempLabel3Heading.setBorder(BorderFactory.createLineBorder(Color.black));

        infoHTMLString.append("<html><table border=\"1\"><tr><td>");
        infoHTMLString.append(" ");
        infoHTMLString.append("</td><td>");
        infoHTMLString.append(tempLabel1Heading.getText());
        infoHTMLString.append("</td><td>");
        infoHTMLString.append(tempLabel2Heading.getText());
        infoHTMLString.append("</td><td>");
        infoHTMLString.append(tempLabel3Heading.getText());
        infoHTMLString.append("</td></tr>");

        infoPanel.add(new JLabel());
        infoPanel.add(tempLabel1Heading);
        infoPanel.add(tempLabel2Heading);
        infoPanel.add(tempLabel3Heading);
        for (String key : measures) {
            JLabel tempLabel = new JLabel();
            tempLabel.setBorder(BorderFactory.createLineBorder(Color.black));
            JLabel tempLabel1 = new JLabel();
            tempLabel1.setBorder(BorderFactory.createLineBorder(Color.black));
            JLabel tempLabel2 = new JLabel();
            tempLabel2.setBorder(BorderFactory.createLineBorder(Color.black));
            JLabel tempLabel3 = new JLabel();
            tempLabel3.setBorder(BorderFactory.createLineBorder(Color.black));
            tempLabel.setText(key + " : ");
            tempLabel1.setText(originalMeasurelist.get(key).toString());
            tempLabel2.setText(result1Measurelist.get(key).toString());
            tempLabel3.setText(result2Measurelist.get(key).toString());

            infoHTMLString.append("<tr><td>");
            infoHTMLString.append(tempLabel.getText());
            infoHTMLString.append("</td><td>");
            infoHTMLString.append(tempLabel1.getText());
            infoHTMLString.append("</td><td>");
            infoHTMLString.append(tempLabel2.getText());
            infoHTMLString.append("</td><td>");
            infoHTMLString.append(tempLabel3.getText());
            infoHTMLString.append("</td></tr>");

            infoPanel.add(tempLabel);
            infoPanel.add(tempLabel1);
            infoPanel.add(tempLabel2);
            infoPanel.add(tempLabel3);
        }
        infoHTMLString.append("</table>");
        infoHTMLString.append("<br>");
        infoHTMLString.append(ResultFileManager.getFile(resultFile1).getFileName());
        infoHTMLString.append(" Description:");
        infoHTMLString.append("<br>");
        infoHTMLString.append(ResultFileManager.getFile(resultFile1).getDescription());
        infoHTMLString.append("<br>");
        infoHTMLString.append(ResultFileManager.getFile(resultFile2).getFileName());
        infoHTMLString.append(" Description:");
        infoHTMLString.append("<br>");
        infoHTMLString.append(ResultFileManager.getFile(resultFile2).getDescription());
        infoHTMLString.append("</html>");
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }

    /**
     * 
     */
    private void saveFile() {
        Document document = new Document();
        final JFileChooser fc = new JFileChooser();
        int returnVal = fc.showSaveDialog(this);
        try {
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                PdfWriter.getInstance(document, new FileOutputStream(file));
                document.open();
                List htmlarraylist = HTMLWorker.parseToList(new StringReader(infoHTMLString.toString()), null);
                //add the collection to the document
                for (int k = 0; k < htmlarraylist.size(); k++) {
                    document.add((Element) htmlarraylist.get(k));
                }
            }
        } catch (DocumentException | IOException e) {
        } finally {
            if (document.isOpen()) {
                document.close();
            }
        }
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == saveToPDFButton) {
            saveFile();
        }
    }
}
