package GUI.TestToolGUI.ManageFilePanes;

import FileManagement.FileTools.AlgorithmFileManager;
import FileManagement.FileTools.AlgorithmFileType;
import GUI.TestToolGUI.FileChooser.GUIFileChooser;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import net.java.dev.designgridlayout.DesignGridLayout;

/**
 *
 * @author Matty
 */
public class NewAlgorithmPane extends JFrame implements ActionListener, Runnable {

    private JPanel panel;
    private final GUIFileChooser fc = new GUIFileChooser();
    //
    private JLabel algorithmNameLabel;
    private JTextField inAlgorithmName;
    private JButton selectFile;
    //
    private JLabel algorithmDescriptionLabel;
    private JTextArea description;
    private JScrollPane descriptionScrollPane;
    //
    private JLabel additionalFolderLabel;
    private JTextField additionalFolderName;
    private JButton additionalFolderButton;
    private JButton addFile, cancel;
    //
    private JLabel fileTypeLabel;
    private JComboBox fileTypeList;
    private final String[] fileTypes = {AlgorithmFileType.RELATIONAL, AlgorithmFileType.TRANSACTIONAL};
    //
    private JLabel numberOfParametersLabel;
    private JComboBox numberOfParametersList;
    private final Integer[] parameterNumbers = {2, 3, 4, 5, 6, 7, 8, 9, 10};
    private Integer numberOfParameters;
    private ArrayList<ArrayList<String>> parameterTypeAndDescription;
    private JButton parameterEntryButton;
    //
    private String inputFileType;
    //
    private Boolean fileChosen;
    private DesignGridLayout layout;
    private File file, additionalFolder;
    private ManageAlgorithmPane parentPane;
    private NewAlgorithmPaneParameterPane entryPane;

    /**
     * 
     * @param inParentPane 
     */
    public NewAlgorithmPane(ManageAlgorithmPane inParentPane) {
        super();
        parentPane = inParentPane;
    }

    /**
     * 
     */
    private void createAndShow() {
        setTitle("Add ALgorithm");
        fileChosen = false;
        panel = new JPanel();
        layout = new DesignGridLayout(panel);
        addEntryBoxes();
        this.add(panel);
    }

    /**
     * 
     */
    private void addEntryBoxes() {
        algorithmNameLabel = new JLabel("Algorithm Name");
        inAlgorithmName = new JTextField(20);
        inAlgorithmName.setEditable(false);
        selectFile = new JButton("Browse");
        selectFile.addActionListener((ActionListener) this);

        additionalFolderLabel = new JLabel("Additional Directory");
        additionalFolderName = new JTextField(20);
        additionalFolderName.setEditable(false);
        additionalFolderButton = new JButton("Browse");
        additionalFolderButton.addActionListener(this);

        algorithmDescriptionLabel = new JLabel("Description");
        description = new JTextArea(10, 10);
        description.setLineWrap(true);
        descriptionScrollPane = new JScrollPane(description);

        fileTypeLabel = new JLabel("Dataset Type");
        fileTypeList = new JComboBox(fileTypes);
        fileTypeList.setSelectedIndex(0);
        inputFileType = fileTypes[0];
        fileTypeList.addActionListener(this);

        numberOfParametersLabel = new JLabel("Number of Parameters");
        numberOfParametersList = new JComboBox(parameterNumbers);
        fileTypeList.setSelectedIndex(0);
        numberOfParameters = parameterNumbers[0];
        numberOfParametersList.addActionListener(this);
        parameterEntryButton = new JButton("Enter Parameter Description");
        parameterEntryButton.addActionListener(this);

        addFile = new JButton("Add");
        cancel = new JButton("Cancel");
        addFile.addActionListener((ActionListener) this);
        cancel.addActionListener((ActionListener) this);

        layout.row().grid().add(algorithmNameLabel).add(inAlgorithmName).add(selectFile);
        layout.row().grid().add(additionalFolderLabel).add(additionalFolderName).add(additionalFolderButton);
        layout.row().grid().add(algorithmDescriptionLabel).add(descriptionScrollPane);
        layout.row().grid().add(fileTypeLabel).add(fileTypeList);
        layout.row().grid().add(numberOfParametersLabel).add(numberOfParametersList);
        layout.row().grid().add(new JLabel("Set parammeters:")).add(parameterEntryButton);
        layout.row().grid().add(addFile).add(cancel);
    }

    /**
     * 
     * @param inList 
     */
    protected void setParameterList(ArrayList<ArrayList<String>> inList) {
        parameterTypeAndDescription = inList;
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == selectFile) {
            fc.setSelectionFilter(GUIFileChooser.ALGORITHMFILTER);
            int returnVal = fc.showOpenDialog(fc.getFileChooserInstance());
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file = fc.getSelectedFile();
                try {
                    inAlgorithmName.setText(file.getName());
                    fileChosen = true;
                } catch (Exception ex) {
                    Logger.getLogger(NewAlgorithmPane.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        if (ae.getSource() == addFile && fileChosen) {
            try {
                AlgorithmFileType newFile = new AlgorithmFileType(inAlgorithmName.getText(), description.getText(), inputFileType, numberOfParameters, parameterTypeAndDescription);
                AlgorithmFileManager.addFileToList(file,additionalFolder, newFile);
                description.setText("");
                inAlgorithmName.setText("");
                fileChosen = false;
                parentPane.updateList();
            } catch (Exception ex) {
                Logger.getLogger(NewDatasetPane.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        if (ae.getSource() == fileTypeList) {
            inputFileType = (String) fileTypeList.getSelectedItem();
        }
        if (ae.getSource() == numberOfParametersList) {
            numberOfParameters = parameterNumbers[numberOfParametersList.getSelectedIndex()];
        }
        if (ae.getSource() == additionalFolderButton) {
            fc.setSelectionFilter(GUIFileChooser.DIRECTORIESFILTER);
            int returnVal = fc.showOpenDialog(fc.getFileChooserInstance());
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                additionalFolder = fc.getSelectedFile();
                try {
                    additionalFolderName.setText(additionalFolder.getName());
                } catch (Exception ex) {
                    Logger.getLogger(NewAlgorithmPane.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        if (ae.getSource() == parameterEntryButton) {
            entryPane = new NewAlgorithmPaneParameterPane(numberOfParameters, this);
            Thread t = new Thread(entryPane);
            t.start();
        }
        if (ae.getSource() == cancel) {
            this.setVisible(false);
            this.dispose();
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }
}
