package GUI.TestToolGUI.ManageFilePanes;

import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.FileExistsException;
import GUI.TestToolGUI.FileChooser.GUIFileChooser;
import GUI.TestToolGUI.ToolBar.Buttons.ManageDatasetButton;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import net.java.dev.designgridlayout.DesignGridLayout;

/**
 *
 * @author Matty
 */
public class NewDatasetPane extends JFrame implements ActionListener, Runnable {

    private JPanel panel;
    private final GUIFileChooser fc = new GUIFileChooser();
    private JLabel fileName, fileDescription;
    private JTextField inFileName;
    private JTextArea description;
    private JScrollPane scrollPane;
    private JButton selectFile, addFile, cancel;
    private JComboBox fileTypeList;
    private String fileType;
    private Boolean fileChosen;
    private DesignGridLayout layout;
    private File file;
    private final char[] delimiters = {',', ' '};
    private final String[] delimitersAsString = {"comma", "space"};
    private char delimiter;
    private JComboBox delimiterComboBox;
    ManageDatasetPane parentPane;
    private JLabel infoLabel;

    /**
     * 
     * @param inParentPane 
     */
    public NewDatasetPane(ManageDatasetPane inParentPane) {
        parentPane = inParentPane;
    }

    /**
     * 
     */
    private void createAndShow() {
        setTitle("Add Dataset");
        fileChosen = false;
        panel = new JPanel();
        layout = new DesignGridLayout(panel);
        addEntryBoxes();
        this.add(panel);
    }

    /**
     * 
     */
    private void addEntryBoxes() {
        fileName = new JLabel("File Name");
        inFileName = new JTextField(20);
        inFileName.setEditable(false);
        selectFile = new JButton("Browse");
        selectFile.addActionListener((ActionListener) this);

        fileDescription = new JLabel("Description");
        description = new JTextArea(10, 10);
        description.setLineWrap(true);
        scrollPane = new JScrollPane(description);

        String[] fileTypes = {DatasetFileType.RELATIONAL, DatasetFileType.TRANSACTIONAL};
        fileTypeList = new JComboBox(fileTypes);
        fileTypeList.setSelectedIndex(0);
        fileType = fileTypes[0];
        fileTypeList.addActionListener(this);

        delimiterComboBox = new JComboBox(delimitersAsString);
        delimiterComboBox.addActionListener(this);
        delimiterComboBox.setSelectedIndex(0);
        delimiter = delimiters[0];

        addFile = new JButton("Add");
        cancel = new JButton("Cancel");
        addFile.addActionListener((ActionListener) this);
        cancel.addActionListener((ActionListener) this);
        
        infoLabel = new JLabel("Please enter dataset details");

        layout.row().grid().add(fileName).add(inFileName).add(selectFile);
        layout.row().grid().add(new JLabel("Delimiter")).add(delimiterComboBox);
        layout.row().grid().add(fileDescription).add(scrollPane);
        layout.row().grid().add(new JLabel("Dataset Type")).add(fileTypeList);
        layout.row().grid().add(addFile).add(cancel);
        layout.row().grid().add(infoLabel);
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == selectFile) {
            fc.setSelectionFilter(GUIFileChooser.DATASETFILTER);
            int returnVal = fc.showOpenDialog(fc.getFileChooserInstance());
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file = fc.getSelectedFile();
                try {
                    inFileName.setText(file.getName());
                    fileChosen = true;
                } catch (Exception ex) {
                    Logger.getLogger(ManageDatasetButton.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        if (ae.getSource() == addFile && fileChosen) {
            try {
                DatasetFileType newFile = new DatasetFileType(inFileName.getText(), delimiter, description.getText(), fileType);
                DatasetFileManager.addFileToList(file, newFile);
                description.setText("");
                inFileName.setText("");
                fileChosen = false;
                parentPane.updateList();
            } catch (IOException ex) {
                Logger.getLogger(NewDatasetPane.class.getName()).log(Level.SEVERE, null, ex);
            } catch (FileExistsException e){
                infoLabel.setText(e.getMessage());
                this.validate();
            }
        }
        if (ae.getSource() == fileTypeList) {
            fileType = (String) fileTypeList.getSelectedItem();
        }
        if (ae.getSource() == delimiterComboBox) {
            delimiter = delimiters[delimiterComboBox.getSelectedIndex()];
        }
        if (ae.getSource() == cancel) {
            this.setVisible(false);
            this.dispose();
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }
}
