package GUI.TestToolGUI.ManageFilePanes;

import MeasurementTools.MeasurementInfoTools.MeasurementInformation;
import MeasurementTools.MeasurementInfoTools.ListOfMeasures;
import FileManagement.FileTools.AnalysisFileManager;
import FileManagement.FileTools.AnalysisFileType;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.ResultFileManager;
import FileManagement.FileTools.ResultFileType;
import GUI.TestToolGUI.FileChooser.GUIFileChooser;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import net.java.dev.designgridlayout.DesignGridLayout;
import org.apache.commons.io.FilenameUtils;

/**
 *
 * @author Matty
 */
public class NewResultPane extends JFrame implements ActionListener, Runnable {

    private JPanel panel;
    private final GUIFileChooser fc = new GUIFileChooser();
    private JLabel fileName, fileDescription, infoBar, timeToRunLabel;
    private JTextField inFileName, timeToRun;
    private JTextArea description;
    private JScrollPane scrollPane;
    private JButton selectFile, addFile, cancel;
    private JComboBox originalFileList;
    private Integer originalFileNum = null;
    private Boolean fileChosen, originalFileChosen;
    private DesignGridLayout layout;
    private File file;
    private final char[] delimiters = {',', ' '};
    private final String[] delimitersAsString = {"comma", "space"};
    private char delimiter;
    private JComboBox delimiterComboBox;
    private ManageResultPane parentPane;

    /**
     * 
     * @param inParentPane 
     */
    public NewResultPane(ManageResultPane inParentPane) {
        parentPane = inParentPane;
    }

    /**
     * 
     */
    private void createAndShow() {
        setTitle("Add Result File");
        fileChosen = false;
        originalFileChosen = false;
        panel = new JPanel();
        layout = new DesignGridLayout(panel);
        addEntryBoxes();
        this.add(panel);
    }

    /**
     * 
     */
    private void addEntryBoxes() {
        fileName = new JLabel("File Name");
        inFileName = new JTextField(20);
        inFileName.setEditable(false);
        selectFile = new JButton("Browse");
        addFile = new JButton("Add");
        cancel = new JButton("Cancel");
        selectFile.addActionListener((ActionListener) this);
        addFile.addActionListener((ActionListener) this);
        cancel.addActionListener((ActionListener) this);
        fileDescription = new JLabel("Description");
        description = new JTextArea(10, 10);
        description.setLineWrap(true);
        scrollPane = new JScrollPane(description);

        delimiterComboBox = new JComboBox(delimitersAsString);
        delimiterComboBox.addActionListener(this);
        delimiterComboBox.setSelectedIndex(0);
        delimiter = delimiters[0];

        originalFileList = new JComboBox(DatasetFileManager.getFileNameAsArray());
        originalFileList.setSelectedIndex(0);
        originalFileNum = null;
        originalFileList.addActionListener(this);
        infoBar = new JLabel();

        timeToRunLabel = new JLabel("Enter time it took to run Algorithm (ms)");
        timeToRun = new JTextField(20);
        layout.row().grid().add(fileName).add(inFileName).add(selectFile);
        layout.row().grid().add(fileDescription).add(scrollPane);
        layout.row().grid().add(new JLabel("Select original file")).add(originalFileList);
        layout.row().grid().add(timeToRunLabel).add(timeToRun);
        layout.row().grid().add(addFile).add(cancel);
    }

    /**
     * 
     * @param ae 
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == selectFile) {
            fc.setSelectionFilter(GUIFileChooser.DATASETFILTER);
            int returnVal = fc.showOpenDialog(fc.getFileChooserInstance());
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file = fc.getSelectedFile();
                inFileName.setText(file.getName());
                fileChosen = true;
            }
        }
        if (ae.getSource() == addFile && fileChosen && originalFileList.getSelectedIndex() != 0) {
            try {
                ResultFileType newFile = new ResultFileType(originalFileNum, inFileName.getText(), delimiter, description.getText());
                int[] temp = new int[1];
                temp[0] = 0;
                MeasurementInformation info = new MeasurementInformation(temp);
                info.addMeasure(ListOfMeasures.TIMETORUN, new Float(timeToRun.getText()));
                String filename = FilenameUtils.removeExtension(newFile.getFileName()) + "analysis.txt";
                AnalysisFileType infoFile = new AnalysisFileType(filename, newFile.getFileNum());
                newFile.setAnalysisFileNumber(infoFile.getFileNum());
                try {
                    info.saveFile(infoFile.getFilePath());
                } catch (IOException ex) {
                }
                AnalysisFileManager.addFileToList(infoFile);

                ResultFileManager.addFileToList(file, newFile);
                description.setText("");
                inFileName.setText("");
                fileChosen = false;
                parentPane.updateList();
            } catch (IOException ex) {
                Logger.getLogger(NewDatasetPane.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        if (ae.getSource() == addFile && originalFileNum == null) {
            infoBar.setText("Please Select Original File");
        }
        if (ae.getSource() == addFile && !fileChosen) {
            infoBar.setText("Please Select A File To Import");
        }
        if (ae.getSource() == originalFileList) {
            int i = originalFileList.getSelectedIndex();
            originalFileNum = DatasetFileManager.getDatasetFileList().get(i - 1).getFileNum();
        }
        if (ae.getSource() == delimiterComboBox) {
            delimiter = delimiters[delimiterComboBox.getSelectedIndex()];
        }
        if (ae.getSource() == cancel) {
            this.setVisible(false);
            this.dispose();
        }
    }

    /**
     * 
     */
    @Override
    public void run() {
        createAndShow();
        this.pack();
        this.setVisible(true);
    }
}
