package MeasurementTools.MeasurementInfoTools;

import MeasurementTools.Measures.UtilityMeasure;
import FileManagement.DataSetTools.CSVTools.CSVTools;
import FileManagement.FileTools.AnalysisFileManager;
import FileManagement.FileTools.AnalysisFileType;
import FileManagement.FileTools.DatasetFileManager;
import FileManagement.FileTools.DatasetFileType;
import FileManagement.FileTools.ResultFileManager;
import FileManagement.FileTools.ResultFileType;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.CountDownLatch;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.io.FilenameUtils;

/**
 *
 * @author Matty
 */
public class CalculateComparisonMeasures implements Runnable {

    private MeasurementInformation resultInfo, originalInfo;
    private int[] QIDs;
    private ArrayList<String> measures;
    private int fileNumber, originalFileNumber;
    private AnalysisFileType originalAnalysisFile, resultAnalysisFile;
    private DatasetFileType originalFile;
    private ResultFileType resultFile;
    private CountDownLatch threadCounter;

    /**
     * 
     * @param inFileNumber
     * @param inOriginalFilenumber
     * @param inMeasures
     * @param inQIDs 
     */
    public CalculateComparisonMeasures(int inFileNumber, int inOriginalFilenumber, ArrayList<String> inMeasures, int[] inQIDs, CountDownLatch inThreadCounter) {
        QIDs = inQIDs;
        measures = inMeasures;
        fileNumber = inFileNumber;
        originalFileNumber = inOriginalFilenumber;
        threadCounter = inThreadCounter;
    }

    /**
     * 
     * @param inFileNumber
     * @param inOriginalFilenumber
     * @param inMeasures 
     */
    public CalculateComparisonMeasures(int inFileNumber, int inOriginalFilenumber, ArrayList<String> inMeasures, CountDownLatch inThreadCounter) {
        measures = inMeasures;
        fileNumber = inFileNumber;
        originalFileNumber = inOriginalFilenumber;
        threadCounter = inThreadCounter;
        
    }

    /**
     * 
     */
    private void calculateMeasures() {
        originalFile = DatasetFileManager.getFile(originalFileNumber);
        resultFile = ResultFileManager.getFile(fileNumber);

        if (originalFile.getAnalysisFileNumber() != -1) {
            originalAnalysisFile = AnalysisFileManager.getAnalysisFile(originalFile.getAnalysisFileNumber());
            originalInfo = new MeasurementInformation(originalAnalysisFile.getFilePath());
            try {
                originalInfo.readMeasures(originalAnalysisFile.getFilePath());
            } catch (IOException ex) {
                Logger.getLogger(CalculateComparisonMeasures.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            originalInfo = new MeasurementInformation(QIDs);
            String filename = FilenameUtils.removeExtension(originalFile.getFileName()) + "analysis.txt";
            originalAnalysisFile = new AnalysisFileType(filename, originalFile.getFileNum());
            originalFile.setAnalysisFileNumber(originalAnalysisFile.getFileNum());
            DatasetFileManager.updateFile(originalFile);
            AnalysisFileManager.addFileToList(originalAnalysisFile);
        }
        if (resultFile.getAnalysisFileNumber() != -1) {            
            resultAnalysisFile = AnalysisFileManager.getAnalysisFile(resultFile.getAnalysisFileNumber());
            resultInfo = new MeasurementInformation(resultAnalysisFile.getFilePath());
            try {
                resultInfo.readMeasures(resultAnalysisFile.getFilePath());
            } catch (IOException ex) {
                Logger.getLogger(CalculateComparisonMeasures.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            resultInfo = new MeasurementInformation(QIDs);
            String filename = FilenameUtils.removeExtension(resultFile.getFileName()) + "analysis.txt";
            resultAnalysisFile = new AnalysisFileType(filename, resultFile.getFileNum());
            resultFile.setAnalysisFileNumber(resultAnalysisFile.getFileNum());
            ResultFileManager.updateFile(resultFile);
            AnalysisFileManager.addFileToList(resultAnalysisFile);
        }



        if (originalFile.getFileType().equals(DatasetFileType.RELATIONAL)) {
            doRelationalMeasures();

        }
        if (originalFile.getFileType().equals(DatasetFileType.TRANSACTIONAL)) {
            doTransactionalMeasures();
        }
    }

    private boolean checkSameQIDsSelected(MeasurementInformation info, int[] inQIDs) {
        StringBuilder string = new StringBuilder(0);
        for (int i = 0; i < inQIDs.length; i++) {
            String temp = Integer.toString(inQIDs[i]);
            string.append(temp);
        }
        if (info.getMeasure(ListOfMeasures.QIDSSELECTED) == Float.parseFloat(string.toString())) {
            return true;
        } else {
            info.setMeasure(ListOfMeasures.QIDSSELECTED, Float.parseFloat(string.toString()));
            return false;
        }
        
    }

    /**
     * 
     */
    private void doRelationalMeasures() {
        for (int i = 0; i < measures.size(); i++) {
            String currentMeasure = measures.get(i);
            switch (currentMeasure) {
                case ListOfMeasures.TIMETORUN:
                    if (!resultInfo.hasMeasure(currentMeasure)) {
                        resultInfo.addMeasure(currentMeasure, 0f);
                    }
                    if (!originalInfo.hasMeasure(currentMeasure)) {
                        originalInfo.addMeasure(currentMeasure, 0f);
                    } 
                    ;
                    break;
                case ListOfMeasures.UTILITYMEASURE:
                    if (!resultInfo.hasMeasure(currentMeasure)) {
                        resultInfo.addMeasure(currentMeasure, UtilityMeasure.returnUtilityMeasure(CSVTools.getCSVAsArray(resultFile.getFilePath(), resultFile.getDelimiter()), QIDs));
                    } else if (resultInfo.hasMeasure(currentMeasure) && !checkSameQIDsSelected(resultInfo,QIDs) ) {
                        resultInfo.setMeasure(currentMeasure, UtilityMeasure.returnUtilityMeasure(CSVTools.getCSVAsArray(resultFile.getFilePath(), resultFile.getDelimiter()), QIDs));
                    }
                    if (!originalInfo.hasMeasure(currentMeasure)) {
                        originalInfo.addMeasure(currentMeasure, UtilityMeasure.returnUtilityMeasure(CSVTools.getCSVAsArray(originalFile.getFilePath(), originalFile.getDelimiter()), QIDs));
                    } else if (originalInfo.hasMeasure(currentMeasure) && !checkSameQIDsSelected(originalInfo,QIDs)) {
                        originalInfo.setMeasure(currentMeasure, UtilityMeasure.returnUtilityMeasure(CSVTools.getCSVAsArray(originalFile.getFilePath(), originalFile.getDelimiter()), QIDs));
                    }
                    ;
                    break;
            }
        }
        try {
            resultInfo.saveFile(resultAnalysisFile.getFilePath());
            originalInfo.saveFile(originalAnalysisFile.getFilePath());
        } catch (IOException ex) {
        }
    }

    /**
     * 
     */
    private void doTransactionalMeasures() {
    }

    public void run() {
        calculateMeasures();
        threadCounter.countDown();
    }
}
